<?php

namespace Platiti\GPWebPayGpeNs\OpenCart;

use Platiti\GPWebPayGpeNs\UniModul\IDatabaseConnection;

use ReflectionMethod;
use Controller;
use DB;
use stdClass;

if (!defined('Platiti\\GPWebPayGpeNs\\UniModul\\ConstUniModulVarPath')) define('Platiti\\GPWebPayGpeNs\\UniModul\\ConstUniModulVarPath', DIR_SYSTEM . '../UniModul');
require_once(__DIR__."/UniModul.php");


class UniOCHelper {
	/**
	 * @param Controller $controller
	 * @param string $route
	 * @param mixed $args
	 * @return string
	 */
	static function link($controller, $route, $args = '') {
		if (class_exists('Url')) {   // OC 1.5+
			if (VERSION >= '2.3') {
				if ($route=='extension/payment') {
					$route = 'extension/extension';
					$args = $args . ($args != '' ? '&' : '') . 'type=payment';
				}
				if (0 === strpos($route, 'payment/')) {
					$route = 'extension/' . $route;
				}
			}
			$url = $controller->url->link($route, $args, 'SSL'); // @phpstan-ignore-line V OpenCart 1.5 je treti parametr string "SSL"|"NOSSL" v novejsich je to bool
			$url = str_replace('&amp;', '&', $url);
			return $url;
		} else {	// OC 1.4-
			$url = HTTPS_SERVER . 'index.php?route='.$route;
			if ($args) {
				$url .= '&' . $args;
			}
			return $url;
		}
	}

	/**
	 * @param Controller $controller
	 * @param string $url
	 * @return void
	 */
	static function redirect($controller, $url) {
		if (VERSION < '2.0') {
			$r = new ReflectionMethod('Controller', 'redirect');
			$r->setAccessible(true);
			$r->invoke($controller, $url);
		} else {
			$controller->response->redirect($url);
		}
	}

	/**
	 * @param Controller $controller
	 * @param mixed $route
	 * @param string $args
	 * @return void
	 */
	static function redirectLink($controller, $route, $args = '') {
		self::redirect($controller, self::link($controller, $route, $args));
	}

	/**
	 * @param Controller $controller
	 * @return string
	 */
	static function getLanguageWoCulture($controller) {
		if (VERSION >= '2.2') {
			$lang = $controller->language->get('code');
		} else {
			$lang = substr($controller->config->get('config_language'), 0, 2);  // jen kod jazyka, z en-gb vytahne jen en
		}
		if ($lang == 'cz') return 'cs';  // oprava caste chyby v nastaveni jazyka eshopu
		else return $lang;
	}

}


class OpenCartDbConnection implements IDatabaseConnection {

	/** @var DB */
	protected $db; //* OpenCart's DB class */

	/**
	 * @param DB $db
	 * @return void
	 */
	public function __construct($db /* OpenCart's DB class */) {
		$GLOBALS['DatabaseConnection_toSql_object'] = $this;
		$this->db = $db;
	}


	/**
	 * @param string $sql
	 * @return ?array<int, array<string, mixed>>
	 */
	public function sqlQuery($sql) {
		/** @var stdClass|true */
		$res = $this->db->query($sql);  // samo vyhodi exception pri chybe, kterou pak zalogujeme v ramci UniErru
		if ($res === true) {
			return null;	// nebyl select
		} else {
			return $res->rows;
		}
	}

	/**
	 * @param string $sql
	 * @return void
	 */
	public function sqlExecute($sql) {
		$this->sqlQuery($sql);
	}

	/** @return int  */
	function getInsertId() {
		return $this->db->getLastId();
	}

	/**
	 * @param mixed $text
	 * @return string
	 */
	function toSql($text) {
		if (is_null($text)) {
			return "null";
		} else {
			return "'".$this->db->escape($text)."'";
		}
	}

}
