<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\OpenCart;

use Platiti\GPWebPayGpeNs\UniModul\CustomerData;
use Platiti\GPWebPayGpeNs\UniModul\OrderStatus;
use Platiti\GPWebPayGpeNs\UniModul\OrderReplyStatus;
use Platiti\GPWebPayGpeNs\UniModul\OrderToPayInfo;
use Platiti\GPWebPayGpeNs\UniModul\UniCartItem;
use Platiti\GPWebPayGpeNs\UniModul\UniCartItemType;
use Platiti\GPWebPayGpeNs\UniModul\UniCustomerStatistics;
use Platiti\GPWebPayGpeNs\UniModul\UniModulConfig;
use Platiti\GPWebPayGpeNs\UniModul\UniModulFactory;
use Platiti\GPWebPayGpeNs\UniModul\UniModul;
use Platiti\GPWebPayGpeNs\UniModul\ConfigInfo;
use Platiti\GPWebPayGpeNs\UniModul\ConfigSetting;
use Platiti\GPWebPayGpeNs\UniModul\PrePayGWInfo;
use Platiti\GPWebPayGpeNs\UniModul\Http;
use Platiti\GPWebPayGpeNs\UniModul\UniErr;

use function Platiti\GPWebPayGpeNs\UniModul\BeginSynchronized;
use function Platiti\GPWebPayGpeNs\UniModul\EndSynchronized;
use function Platiti\GPWebPayGpeNs\UniModul\create_initialize_object;

use Controller;
use Exception;
use Throwable;
use DateTime;
use Registry;

require_once(__DIR__."/../../../../UniOCHelper.php");

class ControllerPaymentUniAdapter extends Controller {

	/** @var string */
	protected $adapterName = 'OpenCart';
	/** @var ?UniModul */
	protected $uniModul;
	/** @var string */
	protected $lowCaseBinderName;
	/** @var ConfigSetting */
	protected $configSetting;

	/**
	 * @param Registry $registry
	 * @param string $uniModulName
	 * @param string $subMethod
	 * @return void
	 */
	public function __construct($registry, $uniModulName = null, $subMethod='') {
		UniErr::run(null, function() use ($registry, $uniModulName, $subMethod) {
			parent::__construct($registry);
			$this->lowCaseBinderName = strtolower($uniModulName).'binder'.strtolower($subMethod);
			$uniFact = new UniModulFactory();
			$configInfo = $uniFact->getConfigInfo($uniModulName, 'en', $subMethod,  $this->adapterName);
			$this->configSetting = $this->getConfigData($configInfo, $uniModulName);
			$this->uniModul = $uniFact->createUniModul($uniModulName, $this->configSetting, $subMethod);
		});
	}

	/**
	 * @param ConfigInfo $configInfo
	 * @param string $uniModulName
	 * @return ConfigSetting
	 */
	function getConfigData($configInfo, $uniModulName) {
		$configData = array();
		foreach ($configInfo->configFields as $configField) {
			$baseBinder = strtolower($uniModulName).'binder';  // bez submetod
			$configData[$configField->name] = $this->config->get((VERSION >=3 ? 'payment_' : '') . $baseBinder.'_'.$configField->name);
		}
		if (isset($GLOBALS['UniModulExtraConfig'][$uniModulName][$_SERVER['HTTP_HOST']])) {
			$configData = $GLOBALS['UniModulExtraConfig'][$uniModulName][$_SERVER['HTTP_HOST']] + $configData;
		}

		$uniModulConfig = new UniModulConfig();
		$uniModulConfig->databaseConnection = new OpenCartDbConnection($this->db);
		$uniModulConfig->uniModulDirUrl = HTTPS_SERVER . "catalog/view/theme/default/image/uniadapter/";
		$uniModulConfig->funcGetCallbackUrl = array($this, 'getCallbackUrl');
		$uniModulConfig->funcProcessReplyStatus = array($this, 'funcProcessReplyStatus');

		//DB_PREFIX ..
		//if (DB_DRIVER != 'mysql') user_error('Eshop pouziva jinou nez mysql databazi');
		$uniModulConfig->adapterName =  $this->adapterName;
		$cfgs = create_initialize_object(ConfigSetting::class, array('configData'=>$configData, 'uniModulConfig'=>$uniModulConfig));
		return $cfgs;
	}


	/**
	 * @param string $callbackName
	 * @param array<mixed> $arguments
	 * @return string
	 */
	function getCallbackUrl($callbackName, $arguments) {
		return UniOCHelper::link($this, 'payment/'.$this->lowCaseBinderName.'/callbackcb', http_build_query(array('_callbackName' => $callbackName) + $arguments));
	}


	/**
	 * @return mixed
	 */
	public function index() {
		return UniErr::run(null, function() {
			$this->language->load('payment/'.$this->getSpecificUniModulString().'uniadapter');
			//$data['text_brand'] = $this->language->get('text_brand');

			$data['button_confirm'] = $this->language->get('button_confirm');
			$data['button_back'] = $this->language->get('button_back');

			$data['action'] = UniOCHelper::link($this, 'payment/'.$this->lowCaseBinderName.'/postback', "dbg_order_id=" . $this->session->data['order_id']);
			if ($this->request->get['route'] != 'checkout/guest_step_3') {
				$data['back'] = UniOCHelper::link($this, 'checkout/payment');
			} else {
				$data['back'] = UniOCHelper::link($this, 'checkout/guest_step_2');
			}

			if (isset($this->uniModul)) { //vopruz kvuli adapteru
				$orderToPayInfo = $this->getOrderToPayInfo();
				$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);
				$data['selectCsPayBrand'] = $prePayGWInfo->selectCsPayBrand;
				$data['selectCsPayBrandTitle'] = $prePayGWInfo->selectCsPayBrandTitle;
				$data['forexMessage'] = $prePayGWInfo->forexMessage;
				$data['minilogo'] = "<span style=\"background:url('https://www.platiti.cz/muzo/minilogo.png')\"></span>";
			}

			//$this->load->model('checkout/order');   //uz se vola v getOrderToPayInfo

			$this->id = 'payment';


			if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/'.$this->getSpecificUniModulString().'uniadapter.tpl')) {
				$this->template = $this->config->get('config_template') . '/template/payment/'.$this->getSpecificUniModulString().'uniadapter.tpl';
			} else if (VERSION >= '2.2'){
				$this->template = 'payment/'.$this->getSpecificUniModulString().'uniadapter.tpl';
			} else {
				$this->template = 'default/template/payment/'.$this->getSpecificUniModulString().'uniadapter.tpl';
			}

			if (!empty($GLOBALS["UniModul_DebugSession"])) {
				$this->uniModul->logger->writeLog('TRACE ADAPTER: Session order_id: ' . var_export($this->session->data['order_id'], true));
				$this->uniModul->logger->writeLog('TRACE ADAPTER: Cookies: ' . var_export($_COOKIE, true));
			}

			if (VERSION < '2.0') {
				$this->data = $data;
				$this->render();
			} else {
				UniErr::begin(E_UNIERR_DEFAULT & ~E_USER_DEPRECATED);
				return UniErr::end($this->load->view($this->template, $data));
			}
		});
	}

	/**
	 * @return PrePayGWInfo
	 */
	public function getPrePayGWInfoForModel() {
		//var_dump($this->session->data); wwww();

		//session->data[order_id] jeste v tuto chvily neexistuje, takze orderInfo z controlleru nelze pouzit
		//$orderToPayInfo = $this->getOrderToPayInfo();

		$total_baseCur = $this->cart->getTotal();
		if ($total_baseCur == 0) $total_baseCur = 0.1; // quick fix enabling payment for buyng coupon not summed by getTotal()
		$total_actcur = $this->currency->format($total_baseCur, $this->session->data['currency'], '', false);   // prevede do aktualni meny

		$orderToPayInfo = new OrderToPayInfo();
		$orderToPayInfo->subMethod = $this->uniModul->subMethod;
		$orderToPayInfo->language = UniOCHelper::getLanguageWoCulture($this);
		$orderToPayInfo->currency = $this->session->data['currency'];
		$orderToPayInfo->currencyRates = $this->getCurrencyRates();
		$orderToPayInfo->amount = $total_actcur;

		$customerData = new CustomerData();

		if (isset($this->session->data['payment_address'])) {	// OC 2.0+
			$payment_address = $this->session->data['payment_address'];
		} else if (isset($this->session->data['guest']['payment'])) {	// OC 1.5  guest
			$payment_address = $this->session->data['guest']['payment'];
		} else if (isset($this->session->data['payment_address_id'])) {
			$this->load->model('account/address');		// OC 1.5  registrovany
			$payment_address = $this->model_account_address->getAddress($this->session->data['payment_address_id']);
		} else {
			$payment_address = null;
		}

		if ($payment_address != null) {
			$customerData->first_name = isset($payment_address['firstname']) ? $payment_address['firstname'] : null;
			$customerData->last_name = isset($payment_address['lastname']) ? $payment_address['lastname'] : null;
			//$customerData->email = $order_info['email'];  // zatim neni pro prepayinfo potreba (pro Agmo staci jen country)
			$customerData->street = (isset($payment_address['address_1']) ? $payment_address['address_1'] : null) .' ' . (isset($payment_address['address_2']) ? $payment_address['address_2'] : null);
			$customerData->city = isset($payment_address['city']) ? $payment_address['city'] : null;
			$customerData->post_code = isset($payment_address['postcode']) ? $payment_address['postcode'] : null;
			$customerData->country = isset($payment_address['iso_code_2']) ? $payment_address['iso_code_2'] : null;
			//$customerData->phone = $order_info['telephone'];
			//$customerData->identifier = $order_info['customer_id'] != 0 ? $order_info['customer_id'] : null;
		}
		$orderToPayInfo->customerData = $customerData;

		$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);
		return $prePayGWInfo;
	}

	/**
	 * @return array<string, float>
	 */
	protected function getCurrencyRates() {
		$this->load->model('localisation/currency');
		/** @var array<string, array<string, mixed>> */
		$currencies = $this->model_localisation_currency->getCurrencies();
		$currencyRates = array();
		foreach ($currencies as $code=>$cur) {
			$currencyRates[$code] = (float)$cur['value'];
		}
		return $currencyRates;
	}

	/**
	 * @return OrderToPayInfo
	 */
	protected function getOrderToPayInfo() {
		$this->load->model('checkout/order');
		if (empty($this->session->data['order_id'])) {
			$message = 'Session lost order_id, Cookies: ' . var_export($_COOKIE, true);
			throw new Exception($message);
		}
		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

		if (VERSION < '1.5') {
			$currency = $order_info['currency'];
			$value = $order_info['value'];
		} else {
			$currency = $order_info['currency_code'];
			$value = $order_info['currency_value'];
		}
		$order_total = $this->currency->format($order_info['total'], $currency, $value, FALSE);


		$orderToPayInfo = new OrderToPayInfo();
		$orderToPayInfo->subMethod = $this->uniModul->subMethod;
		$orderToPayInfo->shopOrderNumber = (string)$this->session->data['order_id'];
		$orderToPayInfo->shopPairingInfo = (string)$this->session->data['order_id'];
		$orderToPayInfo->amount = $order_total;
		$orderToPayInfo->currency = $currency;
		if (isset($_REQUEST['brand'])) {
			$orderToPayInfo->ccBrand = $_REQUEST['brand'];
		}


		$customerData = new CustomerData();
		$customerData->first_name = $order_info['firstname'];
		$customerData->last_name = $order_info['lastname'];
		$customerData->email = $order_info['email'];
		$customerData->street = $order_info['payment_address_1']
			? ($order_info['payment_address_1'].' '.(isset($order_info['payment_address_2']) ? $order_info['payment_address_2'] : null))
			: ($order_info['shipping_address_1'].' '.(isset($order_info['shipping_address_2']) ? $order_info['shipping_address_2'] : null));
		$customerData->city = $order_info['payment_city'] ?: $order_info['shipping_city'];
		$customerData->post_code = $order_info['payment_postcode'] ?: $order_info['shipping_postcode'];
		$customerData->country = $order_info['payment_iso_code_2'] ?: $order_info['shipping_iso_code_2'];
		$customerData->phone = $order_info['telephone'];
		$customerData->identifier = $order_info['customer_id'] != 0 ? $order_info['customer_id'] : null;
		$orderToPayInfo->customerData = $customerData;

		$orderToPayInfo->language = UniOCHelper::getLanguageWoCulture($this);
		$orderToPayInfo->description = $this->config->get('config_name');

		$replyUrl = $replyUrl = UniOCHelper::link($this, 'payment/'.$this->lowCaseBinderName.'/callback');
		$orderToPayInfo->replyUrl = $replyUrl;

		$replyUrl = $replyUrl = UniOCHelper::link($this, 'payment/'.$this->lowCaseBinderName.'/notify');
		$orderToPayInfo->notifyUrl = $replyUrl;

		$orderToPayInfo->uniModulDirUrl = HTTPS_SERVER . "catalog/view/theme/default/image/uniadapter/";


		$orderToPayInfo->currencyRates = $this->getCurrencyRates();



		$cartItems = [];
		foreach ($this->cart->getProducts() as $product) {
			$cartItem = new UniCartItem();
			$cartItem->type = UniCartItemType::commodity;
			$cartItem->name = $product['name'];
			$cartItem->quantity = (float)$product['quantity'];

			$taxes = $this->tax->getRates($product['price'], $product['tax_class_id']);
			$rate = 0;
			foreach ($taxes as $tax) {
				if ($tax['type']!='P') continue;
				if (isset($tax['rate'])) {
					$rate += $tax['rate'];
				} else {
					$rate += round(100 * $tax['amount'] / $product['price']);  // pro OC 1.5
				}
			}
			$cartItem->taxRate = $rate;

			$cartItem->unitPrice = $this->currency->format($product['total'] / $product['quantity'] * (1 + $rate/100), $currency, $value, FALSE);
			$cartItem->unitTaxAmount = $this->currency->format($product['total'] / $product['quantity'], $currency, $value, FALSE) * ($rate/100);
			$cartItems[] = $cartItem;
		}

		if ($this->cart->hasShipping() && isset($this->session->data['shipping_method'])) {
			$cartItem = new UniCartItem();
			$cartItem->type = UniCartItemType::delivery;
			$cartItem->name = 'shipping / doprava';
			$cartItem->quantity = 1;
			$taxes = $this->tax->getRates($this->session->data['shipping_method']['cost'], $this->session->data['shipping_method']['tax_class_id']);
			$rate = 0;
			foreach ($taxes as $tax) {
				if ($tax['type']!='P') continue;
				if (isset($tax['rate'])) {
					$rate += $tax['rate'];
				} else {
					$rate += round(100 * $tax['amount'] / $this->session->data['shipping_method']['cost']);  // pro OC 1.5
				}
			}
			$cartItem->taxRate = $rate;
			$cartItem->unitPrice = $this->currency->format($this->session->data['shipping_method']['cost'] * (1 + $rate/100), $currency, $value, FALSE);
			$cartItem->unitTaxAmount = $this->currency->format($this->session->data['shipping_method']['cost'], $currency, $value, FALSE) * ($rate/100);
			$cartItems[] = $cartItem;
		}

		$rozdeleni = array();
		$celkem = 0;
		foreach ($cartItems as $ci) {
			$act = $ci->unitPrice * $ci->quantity;
			$cum = isset($rozdeleni[$ci->taxRate]) ? $rozdeleni[$ci->taxRate] : 0;
			$rozdeleni[($ci->taxRate)] = $cum + $act;
			$celkem += $ci->unitPrice * $ci->quantity;
		}

		// rovnomerne rozpocteni slevy na pasma
		$discountAmount = $orderToPayInfo->amount - $celkem;
		if ($discountAmount != 0) {
			foreach($rozdeleni as $tax=>$amt) {
				$cartItem = new UniCartItem();
				if ($discountAmount < 0) {
					$cartItem->type = UniCartItemType::discount;
					$cartItem->name = 'discount / sleva';
				} else {
					$cartItem->type = UniCartItemType::delivery;
					$cartItem->name = 'rounding / zaokrouhlení';
				}
				$cartItem->quantity = 1;
				$cartItem->unitPrice = $discountAmount * $amt / $celkem;
				$cartItem->taxRate = $tax;
				$cartItem->unitTaxAmount = $cartItem->unitPrice * ($tax / (100 + $tax));
				$cartItems[] = $cartItem;
			}
		}

		$orderToPayInfo->cartItems = $cartItems;

		//statistiky
		if (VERSION >= '3.0' && $order_info['customer_id'] != 0) {

			$stats = new UniCustomerStatistics();

			$this->load->model('account/customer');
			$customer = $this->model_account_customer->getCustomer($order_info['customer_id']);
			$stats->registeredDate = new DateTime($customer['date_added']);

			$query = $this->db->query("SELECT max(date_added) as lastOrderDate, count(*) as cnt, sum(total) as total, max(total) as maximal
				FROM `" . DB_PREFIX . "order`
				WHERE customer_id = '" . (int)$order_info['customer_id'] . "'
				AND order_status_id in (" . implode(", ", array_map(function ($s) {return "'" . (int)$s . "'";}, $this->config->get('config_complete_status'))) . ")
				AND store_id = '" . (int)$this->config->get('config_store_id')."'
				AND currency_code = '" . $this->db->escape($orderToPayInfo->currency) . "'");

				if ($query->row['cnt'] > 0) {
					$stats->lastOrderDate = new DateTime($query->row['lastOrderDate']);
					$stats->orderCount = $query->row['cnt'];
					$stats->orderTotalSum = $query->row['total'];
					$stats->orderTotalMax = $query->row['maximal'];
					$stats->currency = $orderToPayInfo->currency;
			}

			$orderToPayInfo->customerStatistics = $stats;
		}

		return $orderToPayInfo;
	}

	/**
	 * @return void
	 */
	public function postback() {
		UniErr::run(null, function() {
			if (empty($this->session->data['order_id'])) {
				// V realu se to totiz deje nejak asi z historie browser a realne to platby neblokuje - Platiti#7920074
				// Mozna by se do budoucna dalo kontrolovat ten GET parametr dbg_order_id, který by zjistil zda dana objednavka jiz byla vytvorena a pak by bylo jiste ze to neni problem. Pripadne redirektovat na ok/ nok stranu.
				$message = 'Session lost order_id, Cookies: ' . var_export($_COOKIE, true);
				$this->uniModul->logger->writeLog('ERROR ADAPTER: ' . $message);
				echo "Platnost relace platební brány již vypršela. Pokud si myslíte, že se jedná o chybu, kontaktujte nás prosím.<p>\n";
				echo "The payment gateway session has already expired. If you think this is an error, please contact us.<p>\n";
				die();
			}
			$orderToPayInfo = $this->getOrderToPayInfo();
			$redirectAction = $this->uniModul->gatewayOrderRedirectAction($orderToPayInfo);
			if ($redirectAction->orderReplyStatus != null) {   // okamzita odpoved muze byt i zaroven s redirektem - napr. Cetelem
				$frontend_redir = $redirectAction->redirectUrl==null && $redirectAction->redirectForm==null;
				$this->processReply($redirectAction->orderReplyStatus, $frontend_redir);
			}
			if ($redirectAction->inlineForm) {
				$this->showInFrontEnd($redirectAction->inlineForm);
			} else if ($redirectAction->redirectUrl != null) {
				UniOCHelper::redirect($this, $redirectAction->redirectUrl);
			} else if ($redirectAction->redirectForm != null) {
				$this->uniModul->formRedirect($redirectAction->redirectForm);
			}
		});
	}

	/**
	 * @return void
	 */
	public function callback() {
		UniErr::run(null, function() {
			$language = UniOCHelper::getLanguageWoCulture($this);
			$orderReplyStatus = $this->uniModul->gatewayReceiveReply($language);
			$this->processReply($orderReplyStatus, true);
		});
	}

	/**
	 * @return void
	 */
	public function notify() {
		UniErr::run(null, function() {
			$orderReplyStatus = $this->uniModul->gatewayReceiveNotification();
			$this->processReply($orderReplyStatus, false);
		});
	}

	// skutecny univerzalni callback
	/**
	 * @return void
	 */
	public function callbackcb() {
		UniErr::run(null, function() {
			$this->uniModul->processCallbackRequest(Http::get('_callbackName'), $_REQUEST);
		});
	}


	/**
	 * @param mixed $product_info
	 * @return void
	 */
	public function installmentembedhtml($product_info) {
		UniErr::run(null, function() use ($product_info) {
			if (!$this->config->get((VERSION >=3 ? 'payment_' : '') . $this->lowCaseBinderName.'_status')) { return; }   // geozonu zatim neresim

			if (!isset($product_info['price'])) {  // fix pro OC 2.1
				$product_info = $product_info[0];
			}

			$price = isset($product_info['special']) ? $product_info['special'] : $product_info['price'];
			$product_price = $this->currency->format($this->tax->calculate($price, $product_info['tax_class_id'], $this->config->get('config_tax')), $this->session->data['currency'], '', false);
			$embedHtml = $this->uniModul->ProductGetInstallmentEmbedHtml(HTTPS_SERVER, $this->session->data['currency'], $product_price);
			if ($embedHtml != null) {
				$this->load->model('extension/payment/'.strtolower($this->uniModul->name).'binder');
				$uniadapter_model = $this->registry->get("model_extension_payment_".strtolower($this->uniModul->name)."binder");

				$uniadapter_model->setInstallmentEmbedHtml($embedHtml . "<br/>");
			}
		});
	}


	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @param bool $frontend_redir
	 * @return void
	 */
	public function processReply($orderReplyStatus, $frontend_redir) {
		$cfg = $this->configSetting->configData;
		switch ($orderReplyStatus->orderStatus) {
			case OrderStatus::$successful:
				$newState = $cfg['orderStatusSuccessfull'];
				$guiOk = true;
				break;
			case OrderStatus::$pending:
				$newState = isset($cfg['orderStatusPending']) ? $cfg['orderStatusPending'] : null;
				$guiOk = true;
				break;
			case OrderStatus::$failedFinal:
			case OrderStatus::$failedRetriable;
				$newState = isset($cfg['orderStatusFailed']) ? $cfg['orderStatusFailed'] : null;
				$guiOk = false;
				break;
			case OrderStatus::$invalidReply;
				$newState = null;
				$guiOk = false;
				break;
			default:
				throw new Exception("Unknown order status");
		}

		BeginSynchronized();

		$order_id = $orderReplyStatus->shopPairingInfo;
		$this->load->model('checkout/order');
		$order_info = $this->model_checkout_order->getOrder($order_id);
		if ($orderReplyStatus->orderStatus != OrderStatus::$invalidReply) {

			$comment = $order_info['comment'];
			if ($comment != '') {
				$comment .= "\n\n";
			}
			//$comment .= 'UniAdapter gwOrdnum ='.$orderReplyStatus->gwOrderNumber;
			$language = UniOCHelper::getLanguageWoCulture($this);  // je to potreba i nize, notifikace nevadi
			$comment .= $this->uniModul->dictionary->get('orderDetailGwOrdNum', $language).': '.$orderReplyStatus->gwOrderNumber;
			if ($orderReplyStatus->forexNote !== null) {
				$comment .= "<br/>".$this->uniModul->dictionary->get('forexNoteLabel', $language). ': ' . $orderReplyStatus->forexNote;
			}

			$oldState = $order_info['order_status_id'];
			if ($oldState == 0) {
				if ($orderReplyStatus->orderStatus != OrderStatus::$failedFinal && $orderReplyStatus->orderStatus != OrderStatus::$failedRetriable) {
					if (VERSION < '2.0') {
						$this->model_checkout_order->confirm($order_id, $newState, $comment, TRUE);
					} else {
						UniErr::begin(E_UNIERR_DEFAULT & ~E_USER_DEPRECATED);
						$this->model_checkout_order->addOrderHistory($order_id, $newState, $comment, TRUE);
						UniErr::end();
					}
					$this->cart->clear(); // mazeme kosik hlavne pro immediate status, pro standardni navrat to tak jako tak vola sablona checkout/success, pro notifikacni pripad tato funkce potrebuje znat session $this->session->getId(), ktere aktualne nemame
					$this->uniModul->logger->writeLog("ADAPTER: Creating shop order number ".$order_id.", new order state ".$newState);
				} else {
					$this->uniModul->logger->writeLog("ADAPTER: Won't create new failed order " . $order_id);
				}
			} else {
				$transientStates = array();
				if (isset($cfg['orderStatusPending'])) $transientStates[] = $cfg['orderStatusPending'];
				if (isset($cfg['orderStatusFailed'])) $transientStates[] = $cfg['orderStatusFailed'];
				if ($oldState != $newState  && in_array($oldState, $transientStates)) {
					if (VERSION < '2.0') {
						$this->model_checkout_order->update($order_id, $newState, $comment, TRUE);
					} else {
						$this->model_checkout_order->addOrderHistory($order_id, $newState, $comment, TRUE);
					}
					$this->uniModul->logger->writeLog("ADAPTER: Updating shop order number ".$order_id.", new order state ".$newState." old order state " . $oldState);
				} else {
					$this->uniModul->logger->writeLog("ADAPTER: Refusing update shop order number ".$order_id.", new order state ".$newState." old order state " . $oldState);
				}

			}
		} else {
			$this->uniModul->logger->writeLog("ADAPTER: Ignoring invalid reply");
		}

		EndSynchronized();

		if ($frontend_redir) {
			if ($guiOk) {
				if  (!empty($orderReplyStatus->successHtml)) {
					$this->language->load('checkout/success');
					$this->document->setTitle($this->language->get('heading_title'));
					$suh = "<h1>".$this->language->get('heading_title')."</h1>";
					$suh .= $orderReplyStatus->successHtml;
					$continue = UniOCHelper::link($this, 'common/home');
					$button_continue = $this->language->get('button_continue');
					$suh .= <<< EOT
	<div class="buttons">
		<div class="right"><a href="$continue" class="button"><span>$button_continue</span></a></div>
	</div>
EOT;
					$this->showInFrontEnd($suh);
				} else {
					UniOCHelper::redirectLink($this, 'checkout/success');
				}
			} else {
				$this->language->load('payment/'.$this->getSpecificUniModulString().'uniadapter');
				$errMsg = $this->language->get('message_failure');
				if ($orderReplyStatus->resultText != null) {
					$errMsg .= '<br/>'.$orderReplyStatus->resultText;
				}

				if (VERSION < '1.5') {
					$this->session->data['error'] = $errMsg;
					UniOCHelper::redirectLink($this, 'checkout/payment');

				} else if (VERSION >= '1.5.3' && VERSION < '1.5.5') {
					$this->showFailurePage($errMsg);
				} else {
					$this->session->data['error'] = $errMsg;
					UniOCHelper::redirectLink($this, 'checkout/cart');    // zobrazuje error hlasku v 1.5.1 ale ne v 1.5.3

				}

			}
		}

	}

	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @return void
	 */
	public function funcProcessReplyStatus($orderReplyStatus) {
		$this->processReply($orderReplyStatus, false);
	}

	/**
	 * @param string $errMsg
	 * @return void
	 */
	private function showFailurePage($errMsg) {

			$data['error_warning'] = $errMsg;
			$data['continue_text'] = $this->language->get('button_continue');
			$data['continue_url'] = UniOCHelper::link($this, 'checkout/checkout');

			if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/'.$this->getSpecificUniModulString().'uniadapter_failure.tpl')) {
				$template = $this->config->get('config_template') . '/template/payment/'.$this->getSpecificUniModulString().'uniadapter_failure.tpl';
			} else if (VERSION >= '2.2'){
				$template = 'payment/'.$this->getSpecificUniModulString().'uniadapter_failure.tpl';
			} else {
				$template = 'default/template/payment/'.$this->getSpecificUniModulString().'uniadapter_failure.tpl';
			}

			$this->children = array(
				'common/column_left',
				'common/column_right',
				'common/content_top',
				'common/content_bottom',
				'common/footer',
				'common/header'
			);
			if (VERSION < '2.0') {
				$this->data = $data;
				$this->template = $template;
				$this->response->setOutput($this->render());
			} else {
				throw new Exception("Usable only with older OpenCarts");
			}
	}


	/**
	 * @param string $html
	 * @return void
	 */
	private function showInFrontEnd($html) {

			$data['html_content'] = $html;
			$data['continue_text'] = $this->language->get('button_continue');
			$data['continue_url'] = UniOCHelper::link($this, 'checkout/checkout');

			if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/'.$this->getSpecificUniModulString().'uniadapter_infrontend.tpl')) {
				$template = $this->config->get('config_template') . '/template/payment/'.$this->getSpecificUniModulString().'uniadapter_infrontend.tpl';
			} else if (VERSION >= '2.2'){
				$template = 'payment/'.$this->getSpecificUniModulString().'uniadapter_infrontend.tpl';
			} else {
				$template = 'default/template/payment/'.$this->getSpecificUniModulString().'uniadapter_infrontend.tpl';
			}

			if (VERSION < '2.0') {
				$this->children = array(
					'common/column_left',
					'common/column_right',
					'common/content_top',
					'common/content_bottom',
					'common/footer',
					'common/header'
				);
				$this->data = $data;
				$this->template = $template;
				$this->response->setOutput($this->render());
			} else {
				UniErr::begin(E_UNIERR_DEFAULT & ~E_USER_DEPRECATED);
				$data['column_left'] = $this->load->controller('common/column_left');
				$data['column_right'] = $this->load->controller('common/column_right');
				$data['content_top'] = $this->load->controller('common/content_top');
				$data['content_bottom'] = $this->load->controller('common/content_bottom');
				$data['footer'] = $this->load->controller('common/footer');
				$data['header'] = $this->load->controller('common/header');
				$outhmtl = $this->load->view($template, $data);
				if (VERSION >= '2.0') {
					$this->response->setOutput($outhmtl);
				}
				UniErr::end();
			}
	}

	/**
	 * @return string
	 */
	public function getSpecificUniModulString() {
		if ("GPWebPayGpeNs" != "Pti"."UniNs") { // @phpstan-ignore-line
			return strtolower($this->uniModul->name) . '_';
		} else {
			return '';
		}
	}
}

