<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\UniModul;

use DateTime;
use DateTimeInterface;

/** @deprecated */
class CheckNonexistentFields {
	/**
	 * @param string $name
	 * @param mixed $value
	 * @return void
	 */
	function __set($name, $value ) {
		user_error("Assigning nonexistent field ".get_class($this)."::".$name);
		$this->$name = $value;
	}
}

/** @deprecated */
class CheckNonexistentFieldsLogOnly {
	/**
	 * @param string $name
	 * @param mixed $value
	 * @return void
	 */
	function __set($name, $value ) {
		UniErr::writeErrLog(0, 'CheckNonexistentFieldsLogOnly ' . "Assigning nonexistent field ".get_class($this)."::".$name, null, null, 2);
		$this->$name = $value;
	}
}


class OrderToPayInfo {
	/** @var ?string pouze pokud je to znamo dopredu, jinak null */
	var $shopOrderNumber;
	/** @var string */
	var $shopPairingInfo;
	/** @var mixed serialized to blob, sessions atp */
	var $uniAdapterData;
	/** @var float */
	var $amount;
	/** @var string ciselnik CZK, EUR, USD, GBP */
	var $currency;
	/** @var ?string ciselnik dle CS pro CS, Tato polozka se jiz nepouziva */
	var $ccBrand;
	/** @var CustomerData type CustomerData */
	var $customerData;
	/** @var string ciselnik  cs, sk, en, de, pl - ISO-639
	 * @link https://www.ics.uci.edu/pub/ietf/http/related/iso639.txt */
	var $language;
	/** @var string */
	var $description;
	/** @var string */
	var $replyUrl;
	/** @var string */
	var $notifyUrl;
	/** @var string url adresare UniModul v rootu eshopu,    TODO: Nove uz by se melo brat jen z $this->baseConfig->uniModulDirUrl */
	var $uniModulDirUrl;
	/** @var array<string, float> array(ISO=>rate) */
	var $currencyRates = array();
	/** @var ?string nazev vybrane submetody, prazdne pro defaultni metodu */
	var $subMethod;
	/** @var array<string, mixed> Deprecated - aktualne se nevyuziva, asoc pole vlastnosti/pozadavku Adapteru */
	var $adapterProperties = array();
	/** @var int {@see RecurrenceType} */
	var $recurrenceType = RecurrenceType::none;
	/** @var ?string pro recurrenceType::child je to prislusny ShopOrderNumber s recurrenceType::parent */
	var $recurrenceParentShopOrderNumber;
	/** @var ?string YYYY-MM-DD */
	var $recurrenceDateTo;
	/** @var array<int, UniCartItem> array of CartItem */
	var $cartItems = array();
	/** @var ?UniCustomerStatistics */
	public $customerStatistics;
}

class CustomerData {
	/** @var string */
	var $email;
	/** @var string */
	var $first_name;
	/** @var string */
	var $last_name;
	/** @var string */
	var $street;
	/** @var ?string */
	var $houseNumber;
	/** @var string */
	var $city;
	/** @var string */
	var $post_code;
	/** @var ?string iso AZ, TX - ISO_3166-2
	 * @link http://en.wikipedia.org/wiki/ISO_3166-2:US */
	var $state;
	/** @var string iso CZ, US - ISO_3166-1
	 * @link https://cs.wikipedia.org/wiki/ISO_3166-1 */
	var $country;
	/** @var string */
	var $phone;
	/** @var ?string identifikator zakaznika v ramci eshopu */
	var $identifier;

}

class UniCartItem {
	/** @var string */
	var $name;
	/** @var float s dani */
	var $unitPrice;
	/** @var float */
	var $quantity;
	/** @var float v procentech, 0 až 100 */
	var $taxRate;
	/** @var ?float samotna dan, dodana pozdeji, nektere adaptery ji nemusi doplnovat */
	var $unitTaxAmount;
	/** @var int {@see UniCartItemType} u starsich adapteru nemusi byt dosazovana */
	var $type;
	/** @var string u starsich adapteru nemusi byt dosazovane */
	var $productId = "";
	/** @var ?string */
	var $ean;
	/** @var ?string */
	var $productUrl;
}

abstract class UniCartItemType {
	const commodity = 1;
	const delivery = 2;
	const discount = 3;
}

class UniCustomerStatistics {
	/** @var ?DateTime */
	public $registeredDate;
	/** @var ?DateTime */
	public $lastOrderDate;
	/** @var ?int */
	public $orderCount;
	/** @var ?float */
	public $orderTotalSum;
	/** @var ?float */
	public $orderTotalMax;
	/** @var ?string */
	public $currency;
}

class TransactionRecord {
	/** @var int */
	var $transactionPK;
	/** @var string */
	var $uniModulName;
	/** @var ?string */
	var $gwOrderNumber;
	/** @var ?string druhe cislo transakce na platebni brane, typicky to, co je znamo az po vytvoreni transakce branou */
	var $gwPairingInfo;
	/** @var ?string */
	var $shopOrderNumber;
	/** @var string */
	var $shopPairingInfo;
	/** @var array<string, mixed> */
	var $uniAdapterData;
	/** @var array<string, mixed> */
	var $uniModulData;
	/** @var ?string */
	var $forexNote;
	/** @var int {@see OrderStatus} */
	var $orderStatus;
	/** @var DateTimeInterface */
	var $dateCreated;
	/** @var ?DateTimeInterface */
	var $dateModified;
}

// {{{ Konfigurace

class ConfigSetting {
	/** @var UniModulConfig */
	var $uniModulConfig;
	/** @var array<string, mixed> */
	var $configData;
}

class ConfigInfo {
	/** @var array<int, ConfigField> */
	var $configFields;
}

class ConfigField {
	/** @var string */
	var $name;
	/** @var int {@see ConfigFieldType} */
	var $type;
	/** @var array<int|string, string> array(value=>label) */
	var $choiceItems;
	/** @var string */
	var $label;
	/** @var string */
	var $comment;
}

class ConfigFieldType {
	/** @var int */
	static $text = 1;
	/** @var int */
	static $choice = 2;
	/** @var int */
	static $orderStatus = 3;
	/** @var int */
	static $subMethodsSelection = 4;
	/** @var int */
	static $orderStatusMultiple = 5;
}

class RecurrenceType {
	const none = 0;
	const parent = 1;
	const child = 2;
}

// }}} Konfigurace


class PrePayGWInfo {
	/** @var string */
	var $paymentMethodName;
	/** @var string */
	var $paymentMethodDescription;
	/** @var string */
	var $paymentMethodIconUrl;
	/** @var bool */
	var $isPossible;
	/** @var bool vyber brandu pro CS */
	var $selectCsPayBrand;
	/** @var string text pro nadpis ke kartám */
	var $selectCsPayBrandTitle;
	/** @var ?string info o automatickem prevodu */
	var $forexMessage;
	/** @var ?string pro konverzi */
	var $convertToCurrency;
	/** @var ?float */
	var $convertToCurrencyAmount;
	/** @var ?array<int, string> submetody pouzitelne pro platbu   - pokud null, tak jen hlavni modul, pokud jich je vice tak prazdny retezec znamena hlavni modul */
	var $subMethods;
}


class OrderReplyStatus {
	/** @var ?string melo by stacit jen to pairing info, asi toto odstranit */
	var $shopOrderNumber;
	/** @var string */
	var $shopPairingInfo;
	/** @var ?string */
	var $gwOrderNumber;
	/** @var ?string druhe cislo transakce na platebni brane, typicky to, co je znamo az po vytvoreni transakce branou */
	var $gwPairingInfo;
	/** @var int {@see OrderStatus} */
	var $orderStatus;
	/** @var ?string pouziva se pri neuspechu pro predani detailni chybove hlasky */
	var $resultText;
	/** @var ?string pouziva se pri ok nebo pendingu pro zobrazeni instrukci k offile platbe, zobrazi se na strane s podekovanim za platbu */
	var $successHtml;
	/** @var ?string */
	var $forexNote;
	/** @var mixed serialized from blob */
	var $uniAdapterData;
}

class OrderStatus {
	/** @var int */
	static $initiated = 0;
	/** @var int */
	static $successful = 1;
	/** @var int */
	static $pending = 2;
	/** @var int */
	static $failedRetriable = 3;
	/** @var int */
	static $failedFinal = 4;
	/** @var int muze byt pouzito i pro nic nerikajici stavy, aby se nezpracovali, napr PayU stav 1. */
	static $invalidReply = 5;
	/** @var int */
	static $gwUnpaired = 6;
}

class RedirectAction {
	/** @var ?string bude zadne z nasledujicich tri, nebo vyplneno redirectUrl nebo redirectForm nebo inlineForm, pokud inlineForm tak totez i v redirectForm pro zpetnou kompatibilitu */
	var $redirectUrl;
	/** @var ?string */
	var $redirectForm;
	/** @var ?string */
	var $inlineForm;
	/** @var ?OrderReplyStatus vysledek hned, obvykle kdyz nelze provest platbu, nebo kdyz je nutno vytvorit objednavku soucasne s presmerovanim na branu */
	var $orderReplyStatus;
}

class InfoBoxData {
	/** @var string */
	public $title;
	/** @var string */
	public $image;
	/** @var string */
	public $link;
	/** @var string */
	public $platitiLink;
	/** @var string */
	public $platitiLinkText;
}

class AdapterCapabilities {
	const shopOrderStatusChanged = 'handlesShopOrderStatusChanged';
}
