<?php

namespace Platiti\GPWebPayGpeNs\OpenCart;

use Platiti\GPWebPayGpeNs\UniModul\IDatabaseConnection;
use Platiti\GPWebPayGpeNs\UniModul\SessionInterface;

use ReflectionMethod;
use \Opencart\System\Engine\Controller;
use \Opencart\System\Library\DB;
use stdClass;
use \Opencart\System\Library\Url;

if (!defined('Platiti\\GPWebPayGpeNs\\UniModul\\ConstUniModulVarPath')) define('Platiti\\GPWebPayGpeNs\\UniModul\\ConstUniModulVarPath', DIR_SYSTEM . '../UniModul');
require_once(__DIR__."/UniModul.php");

class UniOCHelper {
	/**
	 * @param Controller $controller
	 * @param string $route
	 * @param mixed $args
	 * @param ?string $function
	 * @return string
	 */
	static function link($controller, $route, $args = '', $function = null) {
		$route = self::numberReplace($route);
		$route = self::addFunctionToPath($route, $function);
		if (VERSION >= '4.0.2.0') {
			$route = str_replace('common/home', 'common/dashboard', $route);
			$route = preg_replace('/^payment\/([^&]+)/', 'extension/$1/payment/$1', $route) ?? $route;
		}
		return $controller->url->link($route, $args, true);
	}

	/**
	 * @param string $path
	 * @param ?string $function
	 * @return string
	 */
	static function addFunctionToPath($path, $function = null) {
		if (isset($function)) {
			if (VERSION >= '4.0.2.0') {
				$path .= "." . $function;
			} else {
				$path .= "%7C" . $function;
			}
		}
		return $path;
	}

	/**
	 * Cannot handle numbers in files, replace numbers with words for numbers
	 *
	 * @param string $string
	 * @return string
	 */
	static function numberReplace($string) {
		$digitStrings = ["Zero", "One", "Two", "Three", "Four", "Five", "Six", "Seven", "Eight", "Nine"];

		$segments = explode("/", $string);
		$result = [];
		foreach ($segments as $segment) {
			$isLower = strtolower($segment) === $segment;
			for ($i=0; $i < 10; $i++) {
				$digitString = $isLower ? strtolower($digitStrings[$i]) : $digitStrings[$i];
				$segment = str_replace((string) $i, $digitString, $segment);
			}
			$result[] = $segment;
		}

		return join("/", $result);
	}

	/**
	 * @param Controller $controller
	 * @param string $url
	 * @return void
	 */
	static function redirect($controller, $url) {
		$controller->response->redirect($url);
	}

	/**
	 * @param Controller $controller
	 * @param mixed $route
	 * @param string $args
	 * @return void
	 */
	static function redirectLink($controller, $route, $args = '') {
		self::redirect($controller, self::link($controller, $route, $args));
	}

	/**
	 * @param Controller $controller
	 * @return string
	 */
	static function getLanguageWoCulture($controller) {
		$lang = explode("-", $controller->config->get('config_language'))[0];
		if ($lang == 'cz') return 'cs';  // oprava caste chyby v nastaveni jazyka eshopu
		else return $lang;
	}

}


class OpenCartDbConnection implements IDatabaseConnection {

	/** @var DB */
	protected $db; //* OpenCart's DB class */

	/**
	 * @param DB $db
	 * @return void
	 */
	public function __construct($db /* OpenCart's DB class */) {
		$GLOBALS['DatabaseConnection_toSql_object'] = $this;
		$this->db = $db;
	}


	/**
	 * @param string $sql
	 * @return ?array<int, array<string, mixed>>
	 */
	public function sqlQuery($sql) {
		/** @var stdClass|true */
		$res = $this->db->query($sql);  // samo vyhodi exception pri chybe, kterou pak zalogujeme v ramci UniErru
		if ($res === true) {
			return null;	// nebyl select
		} else {
			return $res->rows;
		}
	}

	/**
	 * @param string $sql
	 * @return void
	 */
	public function sqlExecute($sql) {
		$this->sqlQuery($sql);
	}

	/** @return int  */
	function getInsertId() {
		return $this->db->getLastId();
	}

	/**
	 * @param mixed $text
	 * @return string
	 */
	function toSql($text) {
		if (is_null($text)) {
			return "null";
		} else {
			return "'".$this->db->escape($text)."'";
		}
	}

}

class OpenCartSession implements SessionInterface {

	/** @var Controller */
	private $controller;

	/**
	 * @param Controller $controller
	 */
	public function __construct($controller) {
		$this->controller = $controller;
	}

	public function get($name, $default = null) {
		return $this->controller->session->data[$name] ?? $default;
	}

	public function set($name, $value) {
		$this->controller->session->data[$name] = $value;
	}

	public function has($name) {
		return isset($this->controller->session->data[$name]);
	}

	public function remove($name) {
		unset($this->controller->session->data[$name]);
	}
}