<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\OpenCart;

use Platiti\GPWebPayGpeNs\UniModul\UniModulConfig;
use Platiti\GPWebPayGpeNs\UniModul\UniModulFactory;
use Platiti\GPWebPayGpeNs\UniModul\UniModul;
use Platiti\GPWebPayGpeNs\UniModul\ConfigInfo;
use Platiti\GPWebPayGpeNs\UniModul\ConfigSetting;
use Platiti\GPWebPayGpeNs\UniModul\ConfigFieldType;
use Platiti\GPWebPayGpeNs\UniModul\UniErr;

use \Opencart\System\Engine\Controller;
use \Opencart\System\Engine\Registry;

use ReflectionClass;
use ReflectionObject;
use Exception;
use Throwable;

require_once(__DIR__."/../../../UniOCHelper.php");

class ControllerPaymentUniAdapterAdmin extends Controller {
	/** @var mixed */
	private $error = array();

	/** @var string */
	protected $adapterName = 'OpenCart';
	/** @var UniModul */
	protected $uniModul;
	/** @var string */
	protected $lowCaseBinderName;
	/** @var string */
	protected $openCartLowCaseBinderName;
	/** @var string */
	protected $lowCaseModulName;
	/** @var string */
	protected $openCartLowCaseModulName;
	/** @var string */
	protected $defaultPath;
	/** @var string */
	protected $langPath;
	/** @var ConfigInfo */
	protected $configInfo;

	/**
	 * @param Registry $registry
	 * @param string $uniModulName
	 * @param ?string $subMethod
	 * @return void
	 */
	public function __construct($registry, $uniModulName = '', $subMethod='') {
		try {
			UniErr::begin();
			global $config;
			parent::__construct($registry);
			$uniFact = new UniModulFactory();
			$this->configInfo = $uniFact->getConfigInfo($uniModulName, UniOCHelper::getLanguageWoCulture($this), $subMethod,  $this->adapterName);
			if (!empty($subMethod)) {
				$this->configInfo->configFields = array();
			}
			$dummyConfigSettings = new ConfigSetting();
			$dummyConfigSettings->uniModulConfig = new UniModulConfig();
			$dummyConfigSettings->uniModulConfig->legacyUniModulVarPaths = [DIR_SYSTEM."/library/UniModul"];
			$dummyConfigSettings->uniModulConfig->databaseConnection = new OpenCartDbConnection($this->db);

			$this->uniModul = $uniFact->createUniModul($uniModulName, $dummyConfigSettings, $subMethod);

			$this->lowCaseBinderName = strtolower($uniModulName).'binder'.strtolower($subMethod ?? "");
			$this->lowCaseModulName = strtolower($uniModulName).'binder';
			$this->openCartLowCaseBinderName = UniOCHelper::numberReplace($this->lowCaseBinderName);
			$this->openCartLowCaseModulName = UniOCHelper::numberReplace($this->lowCaseModulName);
			if ("GPWebPayGpeNs" != "Pti"."UniNs") { // @phpstan-ignore-line
				$this->defaultPath = 'extension/'.$this->openCartLowCaseModulName.'/payment/';
				$this->langPath = 'extension/'.$this->openCartLowCaseModulName.'/payment/';
			} else {
				$this->template->addPath('extension/UniAdapter', DIR_EXTENSION . 'UniAdapter/admin/view/template/');
				$this->language->addPath('extension/UniAdapter', DIR_EXTENSION . 'UniAdapter/admin/language/');
				$this->defaultPath = 'extension/UniAdapter/payment/';
				$this->langPath = 'extension/'.$this->openCartLowCaseModulName.'/payment/';
			}
			UniErr::end();
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}



	/**
	 * @return void
	 */
	public function index() {
		try {
			UniErr::begin();
			$this->load->language('extension/opencart/payment/bank_transfer'); // pro standardni preklady typu poradi, geo zona, atp.
			$this->load->language($this->defaultPath.'uniadapter');
			$this->load->language($this->langPath.$this->openCartLowCaseBinderName);
			$this->document->setTitle($this->language->get('heading_title'));
			$this->load->model('setting/setting');
			if (($this->request->server['REQUEST_METHOD'] == 'POST') && ($this->validate())) {

				// nechce se mi predelavat sablonu
				foreach ($this->request->post as $pkey=>$pval) {
					unset($this->request->post[$pkey]);
					$npkey = 'payment_' . str_replace('uniadapter_', $this->openCartLowCaseBinderName.'_', $pkey);
					$this->request->post[$npkey] = $pval;
				}

				$this->model_setting_setting->editSetting('payment_' . $this->openCartLowCaseBinderName, $this->request->post);
				$this->session->data['success'] = $this->language->get('text_success');
				UniErr::reset();
				$url = UniOCHelper::link($this, 'marketplace/extension','user_token=' . $this->session->data['user_token'] . '&type=payment');

				$this->response->redirect($url);
			}
			// texty obecnych popisku
			$data['heading_title'] = $this->language->get('heading_title');
			$data['text_edit'] = $this->language->get('edit') . ' ' . $this->language->get('text_edit');

			$this->language->set('text_payment', $this->language->get('text_extension'));

			$data['text_enabled'] = $this->language->get('text_enabled');
			$data['text_disabled'] = $this->language->get('text_disabled');
			$data['text_all_zones'] = $this->language->get('text_all_zones');
			$data['text_yes'] = $this->language->get('text_yes');
			$data['text_no'] = $this->language->get('text_no');

			$data['entry_order_status'] = $this->language->get('entry_order_status');
			$data['entry_order_status_pending'] = $this->language->get('entry_order_status_pending');
			$data['entry_order_status_failed'] = $this->language->get('entry_order_status_failed');

			$data['entry_geo_zone'] = $this->language->get('entry_geo_zone');
			$data['entry_status'] = $this->language->get('entry_status');
			$data['entry_sort_order'] = $this->language->get('entry_sort_order');
			$data['button_save'] = $this->language->get('button_save');
			$data['button_cancel'] = $this->language->get('button_cancel');

			$data['tab_general'] = $this->language->get('tab_general');

			// error hlasky
			if (isset($this->error['warning'])) {
				$data['error_warning'] = $this->error['warning'];
			} else {
				$data['error_warning'] = '';
			}

			if (isset($this->error['email'])) {
				$data['error_email'] = $this->error['email'];
			} else {
				$data['error_email'] = '';
			}

			//bread crumbs
			$breadcrumbs = array();

			$breadcrumbs[] = array(
				'href'      => UniOCHelper::link($this, 'common/home', 'user_token=' . $this->session->data['user_token']),
				'text'      => $this->language->get('text_home'),
				'separator' => FALSE
			);

			$breadcrumbs[] = array(
				'href'      => UniOCHelper::link($this, 'marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment'),
				'text'      => $this->language->get('text_payment'),
				'separator' => ' :: '
			);

			$breadcrumbs[] = array(
				'href'      => UniOCHelper::link($this, 'payment/'.$this->lowCaseBinderName, 'user_token=' . $this->session->data['user_token']),
				'text'      => $this->language->get('heading_title'),
				'separator' => ' :: '
			);

			// verze
			$data['breadcrumbs'] = $breadcrumbs;

			// url akcni
			$data['action'] = UniOCHelper::link($this, 'extension/'.$this->lowCaseModulName.'/payment/'.$this->lowCaseBinderName, 'user_token' . '=' . $this->session->data['user_token'], 'save');

			$data['cancel'] = UniOCHelper::link($this, 'marketplace/extension', 'user_token' . '=' . $this->session->data['user_token'] . '&type=payment');


			// moje konfigurace
			$data['configInfo'] = $this->configInfo;  // zobrazene provedu kompletne v templatu
			// jen donactu data
			foreach ($this->configInfo->configFields as $configField) {
				if (isset($this->request->post['uniadapter_'.$configField->name])) {
					$data['uniadapter_'.$configField->name] = $this->request->post['uniadapter_'.$configField->name];
					$data['uniadapter_'][$configField->name] = $this->request->post['uniadapter_'.$configField->name];
				} else {
					$data['uniadapter_'.$configField->name] = $this->config->get('payment_' . $this->openCartLowCaseBinderName.'_'.$configField->name);
					$data['uniadapter_'][$configField->name] = $this->config->get('payment_' . $this->openCartLowCaseBinderName.'_'.$configField->name);
				}
			}



			// texty stavu objednavky
			$this->load->model('localisation/order_status');
			$data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();


			// geo zona, poradi a enable

			if (isset($this->request->post['uniadapter_geo_zone_id'])) {
				$data['uniadapter_geo_zone_id'] = $this->request->post['uniadapter_geo_zone_id'];
			} else {
				$data['uniadapter_geo_zone_id'] = $this->config->get('payment_' . $this->openCartLowCaseBinderName.'_geo_zone_id');
			}

			$this->load->model('localisation/geo_zone');
			$data['geo_zones'] = $this->model_localisation_geo_zone->getGeoZones();

			if (isset($this->request->post['uniadapter_status'])) {
				$data['uniadapter_status'] = $this->request->post['uniadapter_status'];
			} else {
				$data['uniadapter_status'] = $this->config->get('payment_' . $this->openCartLowCaseBinderName.'_status');
			}
			$data['new_status_toggle'] = VERSION >= '4.0.2.0';

			if (VERSION >= '4.0.2.0') {
				$data['entry_submethods'] = "Submethod selection:";
				$data['subMethods'] = array_merge([''], $this->uniModul->getSubMethods());
				$language = UniOCHelper::getLanguageWoCulture($this);
				foreach ($data['subMethods'] as $subMethod) {
					if (isset($this->request->post['uniadapter_submethod_'.$subMethod.'_status'])) {
						$data['subMethods_status'][$subMethod] = $this->request->post['uniadapter_submethod_'.$subMethod.'_status'];
					} else {
						$data['subMethods_status'][$subMethod] = $this->config->get('payment_' . $this->openCartLowCaseBinderName.'_submethod_'.$subMethod.'_status');
					}
					if ($subMethod == '') {
						$data['subMethods_label'][$subMethod] = $this->uniModul->dictionary->get('payment_method_name', $language);
					} else {
						$data['subMethods_label'][$subMethod] = $this->uniModul->dictionary->get('submethod_name_' . $subMethod, $language);
					}
				}
			}

			if (isset($this->request->post['uniadapter_sort_order'])) {
				$data['uniadapter_sort_order'] = $this->request->post['uniadapter_sort_order'];
			} else {
				$data['uniadapter_sort_order'] = $this->config->get('payment_' . $this->openCartLowCaseBinderName.'_sort_order');
			}


			// vyvolani templatu
			$data['header'] = $this->load->controller('common/header');
			$data['column_left'] = $this->load->controller('common/column_left');
			$data['footer'] = $this->load->controller('common/footer');

			$class = new ReflectionClass(ConfigFieldType::class);
			$data['ConfigFieldType'] = $class->getStaticProperties();
			$this->response->setOutput($this->load->view($this->defaultPath.'uniadapter', $data));
			UniErr::end();
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}


	/** @return bool  */
	private function validate() {
		if (!$this->user->hasPermission('modify', 'extension/'.$this->openCartLowCaseModulName.'/payment/'.$this->openCartLowCaseBinderName)) {
			$this->error['warning'] = $this->language->get('error_permission');
		}

		if (!$this->error) {
			$this->uniModul->moduleInstallVarDir();
			$this->uniModul->moduleInstallDatabase();
			return TRUE;
		} else {
			return FALSE;
		}
	}
	public function save(): void {
		$this->load->language('extension/payment');

		$json = [];

		if (!$this->user->hasPermission('modify', 'extension/'.$this->openCartLowCaseModulName.'/payment/'.$this->openCartLowCaseBinderName)) {
			$json['error'] = $this->language->get('error_permission');
		} else {
			$this->uniModul->moduleInstallVarDir();
			$this->uniModul->moduleInstallDatabase();
			$this->load->model('setting/setting');
			$settings = [];
			foreach ($this->request->post as $k => $v) {
				$settings[str_replace("uniadapter", 'payment_'.$this->openCartLowCaseBinderName, $k)] = $v;
			}

			# add subMethodSelection
			$subMethodsSelection = [];
			foreach (array_merge([''], $this->uniModul->getSubMethods()) as $subMethod) {
				if ($this->request->post['uniadapter_submethod_'.$subMethod.'_status']) {
					$subMethodsSelection[] = $subMethod;
				}
			}
			$settings['payment_' . $this->openCartLowCaseBinderName . '_subMethodsSelection'] = join(" ", $subMethodsSelection);

			$this->model_setting_setting->editSetting('payment_'.$this->openCartLowCaseBinderName, $settings);
			$json['success'] = $this->language->get('text_success');
		}

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json) ?: "{}");
	}

	/**
	 * @return void
	 */
	public function install() {
		$this->uniModul->moduleInstallVarDir();
		$this->uniModul->moduleInstallDatabase();

		$this->load->model('setting/event');

		$this->model_setting_event->deleteEventByCode($this->openCartLowCaseBinderName."_installmentwindow");

		$this->model_setting_event->addEvent(['code' => $this->openCartLowCaseBinderName."_installmentwindow", 'description' => '', 'trigger' => 'catalog/view/product/product/after', 'action' => UniOCHelper::addFunctionToPath('extension/'.$this->openCartLowCaseModulName.'/payment/'.$this->lowCaseBinderName, 'installmentembedhtml'), 'status' => true, 'sort_order' => 1]);
	}

	/**
	 * @return void
	 */
	public function uninstall() {
		$this->load->model('setting/event');
		$this->model_setting_event->deleteEventByCode($this->openCartLowCaseBinderName."_installmentwindow");
	}
}
