<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\OpenCart;

use Platiti\GPWebPayGpeNs\UniModul\CustomerData;
use Platiti\GPWebPayGpeNs\UniModul\OrderStatus;
use Platiti\GPWebPayGpeNs\UniModul\OrderReplyStatus;
use Platiti\GPWebPayGpeNs\UniModul\OrderToPayInfo;
use Platiti\GPWebPayGpeNs\UniModul\UniCartItem;
use Platiti\GPWebPayGpeNs\UniModul\UniCartItemType;
use Platiti\GPWebPayGpeNs\UniModul\UniCustomerStatistics;
use Platiti\GPWebPayGpeNs\UniModul\UniModulConfig;
use Platiti\GPWebPayGpeNs\UniModul\UniModulFactory;
use Platiti\GPWebPayGpeNs\UniModul\UniModul;
use Platiti\GPWebPayGpeNs\UniModul\ConfigInfo;
use Platiti\GPWebPayGpeNs\UniModul\ConfigSetting;
use Platiti\GPWebPayGpeNs\UniModul\PrePayGWInfo;
use Platiti\GPWebPayGpeNs\UniModul\UniErr;

use function Platiti\GPWebPayGpeNs\UniModul\BeginSynchronized;
use function Platiti\GPWebPayGpeNs\UniModul\EndSynchronized;
use function Platiti\GPWebPayGpeNs\UniModul\create_initialize_object;

use \Opencart\System\Engine\Controller;
use \Opencart\System\Engine\Registry;
use Exception;
use Throwable;
use DateTime;

require_once(__DIR__."/../../../UniOCHelper.php");

class ControllerPaymentUniAdapter extends Controller {

	/** @var string */
	protected $adapterName = 'OpenCart';
	/** @var UniModul */
	protected $uniModul;
	/** @var string */
	protected $lowCaseBinderName;
	/** @var string */
	protected $openCartLowCaseBinderName;
	/** @var string */
	protected $lowCaseModulName;
	/** @var string */
	protected $openCartLowCaseModulName;
	/** @var string */
	protected $defaultPath;
	/** @var string */
	protected $uniModulUrl;

	/**
	 * @param Registry $registry
	 * @param string $uniModulName
	 * @param string $subMethod
	 * @return void
	 */
	public function __construct($registry, $uniModulName = '', $subMethod='') {
		try {
			UniErr::begin();
			parent::__construct($registry);
			if (VERSION >= '4.0.2.0') {
				$this->lowCaseBinderName = strtolower($uniModulName).'binder';
				$this->lowCaseModulName = strtolower($uniModulName).'binder';
			} else {
				$this->lowCaseBinderName = strtolower($uniModulName).'binder'.strtolower($subMethod);
				$this->lowCaseModulName = strtolower($uniModulName).'binder';
			}
			$this->openCartLowCaseBinderName = UniOCHelper::numberReplace($this->lowCaseBinderName);
			$this->openCartLowCaseModulName = UniOCHelper::numberReplace($this->lowCaseModulName);

			if ("GPWebPayGpeNs" != "Pti"."UniNs") { // @phpstan-ignore-line
				$this->defaultPath = 'extension/'.$this->openCartLowCaseModulName.'/payment/';
				$this->uniModulUrl = HTTP_SERVER . "extension/".$this->openCartLowCaseModulName."/catalog/view/image/uniadapter/";
			} else {
				$this->template->addPath('extension/UniAdapter', DIR_EXTENSION . 'UniAdapter/catalog/view/template/');
				$this->language->addPath('extension/UniAdapter', DIR_EXTENSION . 'UniAdapter/catalog/language/');
				$this->defaultPath = 'extension/UniAdapter/payment/';
				$this->uniModulUrl = HTTP_SERVER . "extension/UniAdapter/system/library/UniModul/";
			}
			$uniFact = new UniModulFactory();
			$configInfo = $uniFact->getConfigInfo($uniModulName, 'en', $subMethod, $this->adapterName);
			$this->configSetting = $this->getConfigData($configInfo, $uniModulName);
			$this->uniModul = $uniFact->createUniModul($uniModulName, $this->configSetting, $subMethod);
			if (VERSION >= "4.0.2.0") {
				if (isset($this->session->data['payment_method']['code'])) {
					$paymentMethodData = explode(".", $this->session->data['payment_method']['code']);
					$parsedSubMethod = explode("binder", $paymentMethodData[1])[1];
					if ($parsedSubMethod != "") {
						foreach ($this->uniModul->getSubMethods() as $subMethod) {
							if (strtolower($subMethod) == $parsedSubMethod) {
								break;
							}
						}
					} else {
						$subMethod = "";
					}
					$this->uniModul->subMethod = $subMethod;
				}
			}
			UniErr::end();
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}

	/**
	 * @param ConfigInfo $configInfo
	 * @param string $uniModulName
	 * @return ConfigSetting
	 */
	function getConfigData($configInfo, $uniModulName) {
		$configData = array();
		foreach ($configInfo->configFields as $configField) {
			$configData[$configField->name] = $this->config->get('payment_' . $this->openCartLowCaseModulName.'_'.$configField->name);
		}
		if (isset($GLOBALS['UniModulExtraConfig'][$uniModulName][$_SERVER['HTTP_HOST']])) {
			$configData = $GLOBALS['UniModulExtraConfig'][$uniModulName][$_SERVER['HTTP_HOST']] + $configData;
		}

		$uniModulConfig = new UniModulConfig();
		$uniModulConfig->databaseConnection = new OpenCartDbConnection($this->db);
		$uniModulConfig->uniModulDirUrl = $this->uniModulUrl;
		$uniModulConfig->funcGetCallbackUrl = array($this, 'getCallbackUrl');
		$uniModulConfig->funcProcessReplyStatus = array($this, 'funcProcessReplyStatus');
		$uniModulConfig->session = new OpenCartSession($this);

		$uniModulConfig->adapterName = $this->adapterName;
		$cfgs = create_initialize_object(ConfigSetting::class, array('configData'=>$configData, 'uniModulConfig'=>$uniModulConfig));
		return $cfgs;
	}


	/**
	 * @param string $callbackName
	 * @param array<mixed> $arguments
	 * @return string
	 */
	function getCallbackUrl($callbackName, $arguments) {
		return UniOCHelper::link($this, 'extension/'.$this->lowCaseModulName.'/payment/'.$this->lowCaseBinderName, http_build_query(array('_callbackName' => $callbackName) + $arguments), "callbackcb");
	}


	/**
	 * @return mixed
	 */
	public function index() {
		try {
			UniErr::begin();
			$this->load->language($this->defaultPath.'uniadapter');
			//$data['text_brand'] = $this->language->get('text_brand');

			$data['button_confirm'] = $this->language->get('button_confirm');
			$data['button_back'] = $this->language->get('button_back');

			$data['action'] = UniOCHelper::link($this, 'extension/'.$this->lowCaseModulName.'/payment/'.$this->lowCaseBinderName, '', 'postback');
			if ($this->request->get['route'] != 'checkout/guest_step_3') {
				$data['back'] = UniOCHelper::link($this, 'checkout/payment');
			} else {
				$data['back'] = UniOCHelper::link($this, 'checkout/guest_step_2');
			}

			$orderToPayInfo = $this->getOrderToPayInfo();
			$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);
			$data['selectCsPayBrand'] = $prePayGWInfo->selectCsPayBrand;
			$data['selectCsPayBrandTitle'] = $prePayGWInfo->selectCsPayBrandTitle;
			$data['forexMessage'] = $prePayGWInfo->forexMessage;
			$data['minilogo'] = "<span style=\"background:url('https://www.platiti.cz/muzo/minilogo.png')\"></span>";

			return UniErr::end($this->load->view($this->defaultPath.'uniadapter', $data));
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}

	/**
	 * @return PrePayGWInfo
	 */
	public function getPrePayGWInfoForModel() {
		$total_baseCur = $this->cart->getTotal();
		if ($total_baseCur == 0) $total_baseCur = 0.1; // quick fix enabling payment for buyng coupon not summed by getTotal()
		$total_actcur = $this->currency->format($total_baseCur, $this->session->data['currency'], 0, false);   // prevede do aktualni meny

		$orderToPayInfo = new OrderToPayInfo();
		$orderToPayInfo->subMethod = $this->uniModul->subMethod;
		$orderToPayInfo->language = UniOCHelper::getLanguageWoCulture($this);
		$orderToPayInfo->currency = $this->session->data['currency'];
		$orderToPayInfo->currencyRates = $this->getCurrencyRates();
		$orderToPayInfo->amount = (float)$total_actcur;

		$customerData = new CustomerData();

		$customer = $this->session->data['customer'];
		$shipping_address = $this->session->data['shipping_address'] ?? null;
		$payment_address = $this->session->data['payment_address'] ?? $shipping_address;

		$customerData->first_name = $customer['firstname'];
		$customerData->last_name = $customer['lastname'];
		$customerData->email = $customer['email'];
		$customerData->phone = $customer['telephone'];
		$customerData->identifier = $customer['customer_id'] != 0 ? $customer['customer_id'] : null;

		if ($payment_address != null) {
			$customerData->street = $payment_address['address_1'] .' ' . $payment_address['address_2'];
			$customerData->city = $payment_address['city'];
			$customerData->post_code = $payment_address['postcode'];
			$customerData->country = $payment_address['iso_code_2'];
		}
		$orderToPayInfo->customerData = $customerData;

		$prePayGWInfo = $this->uniModul->queryPrePayGWInfo($orderToPayInfo);
		return $prePayGWInfo;
	}

	/**
	 * @return array<string, float>
	 */
	protected function getCurrencyRates() {
		$this->load->model('localisation/currency');
		/** @var array<string, array<string, mixed>> */
		$currencies = $this->model_localisation_currency->getCurrencies();
		$currencyRates = array();
		foreach ($currencies as $code=>$cur) {
			$currencyRates[$code] = (float)$cur['value'];
		}
		return $currencyRates;
	}

	/**
	 * @return OrderToPayInfo
	 */
	protected function getOrderToPayInfo() {
		$this->load->model('checkout/order');
		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

		$currency = $order_info['currency_code'];
		$value = $order_info['currency_value'];

		$order_total = $this->currency->format($order_info['total'], $currency, $value, FALSE);

		$orderToPayInfo = new OrderToPayInfo();
		$orderToPayInfo->subMethod = $this->uniModul->subMethod;
		$orderToPayInfo->shopOrderNumber = (string)$this->session->data['order_id'];
		$orderToPayInfo->shopPairingInfo = (string)$this->session->data['order_id'];
		$orderToPayInfo->amount = (float)$order_total;
		$orderToPayInfo->currency = $currency;
		if (isset($_REQUEST['brand'])) {
			$orderToPayInfo->ccBrand = $_REQUEST['brand'];
		}

		$customerData = new CustomerData();
		$customerData->first_name = $order_info['firstname'];
		$customerData->last_name = $order_info['lastname'];
		$customerData->email = $order_info['email'];
		$customerData->street = $order_info['payment_address_1']
			? ($order_info['payment_address_1'].' '.(isset($order_info['payment_address_2']) ? $order_info['payment_address_2'] : null))
			: ($order_info['shipping_address_1'].' '.(isset($order_info['shipping_address_2']) ? $order_info['shipping_address_2'] : null));
		$customerData->city = $order_info['payment_city'] ?: $order_info['shipping_city'];
		$customerData->post_code = $order_info['payment_postcode'] ?: $order_info['shipping_postcode'];
		$customerData->country = $order_info['payment_iso_code_2'] ?: $order_info['shipping_iso_code_2'];
		$customerData->phone = $order_info['telephone'];
		$customerData->identifier = $order_info['customer_id'] != 0 ? $order_info['customer_id'] : null;
		$orderToPayInfo->customerData = $customerData;

		$orderToPayInfo->language = UniOCHelper::getLanguageWoCulture($this);
		$orderToPayInfo->description = $this->config->get('config_name');

		$replyUrl = UniOCHelper::link($this, 'extension/'.$this->lowCaseModulName.'/payment/'.$this->lowCaseBinderName, '', 'callback');
		$orderToPayInfo->replyUrl = $replyUrl;

		$replyUrl = UniOCHelper::link($this, 'extension/'.$this->lowCaseModulName.'/payment/'.$this->lowCaseBinderName, '', 'notify');
		$orderToPayInfo->notifyUrl = $replyUrl;

		$orderToPayInfo->uniModulDirUrl = $this->uniModulUrl;

		$orderToPayInfo->currencyRates = $this->getCurrencyRates();

		$cartItems = [];
		foreach ($this->cart->getProducts() as $product) {
			$cartItem = new UniCartItem();
			$cartItem->type = UniCartItemType::commodity;
			$cartItem->name = $product['name'];
			$cartItem->quantity = (float)$product['quantity'];

			$taxes = $this->tax->getRates($product['price'], $product['tax_class_id']);
			$rate = 0;
			foreach ($taxes as $tax) {
				if ($tax['type']!='P') continue;
				if (isset($tax['rate'])) {
					$rate += $tax['rate'];
				}
			}
			$cartItem->taxRate = $rate;

			$cartItem->unitPrice = (float)$this->currency->format($product['total'] / $product['quantity'] * (1 + $rate/100), $currency, $value, FALSE);
			$cartItem->unitTaxAmount = (float)($this->currency->format($product['total'] / $product['quantity'], $currency, $value, FALSE) * ($rate/100));
			$cartItems[] = $cartItem;
		}

		if ($this->cart->hasShipping() && isset($this->session->data['shipping_method'])) {
			$cartItem = new UniCartItem();
			$cartItem->type = UniCartItemType::delivery;
			$cartItem->name = 'shipping / doprava';
			$cartItem->quantity = 1;

			if (VERSION >= '4.0.2.0') {
				$shipping = explode('.', $this->session->data['shipping_method']['code']);
			} else {
				$shipping = explode('.', $this->session->data['shipping_method']);
			}

			if (isset($shipping[0]) && isset($shipping[1]) && isset($this->session->data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]])) {
				$shipping_method_info = $this->session->data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]];

				$shipping_method_cost = $shipping_method_info['cost'];

				$taxes = $this->tax->getRates($shipping_method_cost, $shipping_method_info['tax_class_id']);
				$rate = 0;
				foreach ($taxes as $tax) {
					if ($tax['type']!='P') continue;
					if (isset($tax['rate'])) {
						$rate += $tax['rate'];
					}
				}
			} else {
				$rate = 0;
				$shipping_method_cost = 0;
			}


			$cartItem->taxRate = $rate;
			$cartItem->unitPrice = (float)$this->currency->format($shipping_method_cost * (1 + $rate/100), $currency, $value, FALSE);
			$cartItem->unitTaxAmount = (float)$this->currency->format($shipping_method_cost * ($rate / 100), $currency, $value, FALSE);
			$cartItems[] = $cartItem;
		}

		$rozdeleni = array();
		$celkem = 0;
		foreach ($cartItems as $ci) {
			$act = $ci->unitPrice * $ci->quantity;
			$cum = isset($rozdeleni[$ci->taxRate]) ? $rozdeleni[$ci->taxRate] : 0;
			$rozdeleni[($ci->taxRate)] = $cum + $act;
			$celkem += $ci->unitPrice * $ci->quantity;
		}

		// rovnomerne rozpocteni slevy na pasma
		$discountAmount = $orderToPayInfo->amount - $celkem;
		if ($discountAmount != 0) {
			foreach($rozdeleni as $tax=>$amt) {
				$cartItem = new UniCartItem();
				if ($discountAmount < 0) {
					$cartItem->type = UniCartItemType::discount;
					$cartItem->name = 'discount / sleva';
				} else {
					$cartItem->type = UniCartItemType::delivery;
					$cartItem->name = 'rounding / zaokrouhlení';
				}
				$cartItem->quantity = 1;
				$cartItem->unitPrice = $discountAmount * $amt / $celkem;
				$cartItem->taxRate = (float)$tax;
				$cartItem->unitTaxAmount = (float)($cartItem->unitPrice * ($tax / (100 + $tax)));
				$cartItems[] = $cartItem;
			}
		}

		$orderToPayInfo->cartItems = $cartItems;

		if ($order_info['customer_id'] != 0) { # only for non guest customers
			$stats = new UniCustomerStatistics();

			$this->load->model('account/customer');
			$customer = $this->model_account_customer->getCustomer($order_info['customer_id']);
			$stats->registeredDate = new DateTime($customer['date_added']);

			$query = $this->db->query("SELECT max(date_added) as lastOrderDate, count(*) as cnt, sum(total) as total, max(total) as maximal
				FROM `" . DB_PREFIX . "order`
				WHERE customer_id = '" . (int)$order_info['customer_id'] . "'
				AND order_status_id in (" . implode(", ", array_map(function ($s) {return "'" . (int)$s . "'";}, $this->config->get('config_complete_status'))) . ")
				AND store_id = '" . (int)$this->config->get('config_store_id')."'
				AND currency_code = '" . $this->db->escape($orderToPayInfo->currency) . "'");

				if ($query->row['cnt'] > 0) {
					$stats->lastOrderDate = new DateTime($query->row['lastOrderDate']);
					$stats->orderCount = $query->row['cnt'];
					$stats->orderTotalSum = $query->row['total'];
					$stats->orderTotalMax = $query->row['maximal'];
					$stats->currency = $orderToPayInfo->currency;
			}

			$orderToPayInfo->customerStatistics = $stats;
		}

		return $orderToPayInfo;
	}

	/**
	 * @return void
	 */
	public function postback() {
		try {
			UniErr::begin();
			$orderToPayInfo = $this->getOrderToPayInfo();
			$redirectAction = $this->uniModul->gatewayOrderRedirectAction($orderToPayInfo);
			if ($redirectAction->orderReplyStatus != null) {   // okamzita odpoved muze byt i zaroven s redirektem - napr. Cetelem
				$frontend_redir = $redirectAction->redirectUrl==null && $redirectAction->redirectForm==null;
				$this->processReply($redirectAction->orderReplyStatus, $frontend_redir);
			}
			if ($redirectAction->inlineForm) {
				$this->showInFrontEnd($redirectAction->inlineForm);
			} else if ($redirectAction->redirectUrl != null) {
				UniErr::reset();
				UniOCHelper::redirect($this, $redirectAction->redirectUrl);
			} else if ($redirectAction->redirectForm != null) {
				$this->uniModul->formRedirect($redirectAction->redirectForm);
			}
			UniErr::end();
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}

	/**
	 * @return void
	 */
	public function callback() {
		try {
			UniErr::begin();
			$sessionOk = isset($_COOKIE["OCSESSID"]);
			$this->uniModul->webTools->crossPostFix($sessionOk);
			$language = UniOCHelper::getLanguageWoCulture($this);
			$orderReplyStatus = $this->uniModul->gatewayReceiveReply($language);
			$this->processReply($orderReplyStatus, true);
			UniErr::end();
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}

	/**
	 * @return void
	 */
	public function notify() {
		try {
			UniErr::begin();
			$orderReplyStatus = $this->uniModul->gatewayReceiveNotification();
			$this->processReply($orderReplyStatus, false);
			UniErr::end();
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}

	// skutecny univerzalni callback
	/**
	 * @return void
	 */
	public function callbackcb() {
		try {
			UniErr::begin();
			$this->uniModul->processCallbackRequest($_GET['_callbackName'], $_REQUEST);
			UniErr::end();
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}


	/**
	 * @param string $route
	 * @param mixed[] $data
	 * @param string $output
	 * @return void
	 */
	public function installmentembedhtml(string &$route, array &$data, &$output) {
		try {
			UniErr::begin();
			$this->load->model('catalog/product');
			$product = $this->model_catalog_product->getProduct($data["product_id"]);
			$product_price = $this->tax->calculate($product["price"], $product["tax_class_id"]);
			$end_head = strpos($output, "</head>") ?: 0;

			$embedHtml = $this->uniModul->ProductGetInstallmentEmbedHtml(null, $this->session->data['currency'], $product_price);

			$name = $this->openCartLowCaseBinderName;

			# expects a button with id 'button-cart' to exist, will fail otherwise
			$add_text = <<<EOD
			<script>
				window.addEventListener("load", () => {
					let div = document.createElement("div")
					div.id = "uniadapter_installmentembedhtml_$name"
					div.innerHTML = `$embedHtml<br />`

					let button = document.getElementById("button-cart")
					button.parentElement.insertBefore(div, button)
				}, false)
			</script>
			EOD;
			$output = substr_replace($output, $add_text, $end_head, 0);
			UniErr::end();
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}


	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @param bool $frontend_redir
	 * @return void
	 */
	public function processReply($orderReplyStatus, $frontend_redir) {
		$clear_cart = function () {
			$this->cart->clear();
			if (isset($this->session->data['order_id'])) {
				unset($this->session->data['order_id']);
				unset($this->session->data['payment_address']);
				unset($this->session->data['payment_method']);
				unset($this->session->data['payment_methods']);
				unset($this->session->data['shipping_address']);
				unset($this->session->data['shipping_method']);
				unset($this->session->data['shipping_methods']);
				unset($this->session->data['comment']);
				unset($this->session->data['coupon']);
				unset($this->session->data['reward']);
				unset($this->session->data['voucher']);
				unset($this->session->data['vouchers']);
			}
		};

		$cfg = $this->configSetting->configData;
		switch ($orderReplyStatus->orderStatus) {
			case OrderStatus::$successful:
				$newState = $cfg['orderStatusSuccessfull'];
				$guiOk = true;
				break;
			case OrderStatus::$pending:
				$newState = isset($cfg['orderStatusPending']) ? $cfg['orderStatusPending'] : null;
				$guiOk = true;
				break;
			case OrderStatus::$failedFinal:
			case OrderStatus::$failedRetriable;
				$newState = isset($cfg['orderStatusFailed']) ? $cfg['orderStatusFailed'] : null;
				$guiOk = false;
				break;
			case OrderStatus::$invalidReply;
				$newState = null;
				$guiOk = false;
				break;
			default:
				var_export([$orderReplyStatus, $cfg, $orderReplyStatus->orderStatus == OrderStatus::$invalidReply]);
				throw new Exception("Unknown order status");
		}

		BeginSynchronized();

		$order_id = $orderReplyStatus->shopPairingInfo;
		$this->load->model('checkout/order');
		$order_info = $this->model_checkout_order->getOrder($order_id);
		if ($orderReplyStatus->orderStatus != OrderStatus::$invalidReply) {
			$comment = $order_info['comment'];
			if ($comment != '') {
				$comment .= "\n\n";
			}
			//$comment .= 'UniAdapter gwOrdnum ='.$orderReplyStatus->gwOrderNumber;
			$language = UniOCHelper::getLanguageWoCulture($this);  // je to potreba i nize, notifikace nevadi
			$comment .= $this->uniModul->dictionary->get('orderDetailGwOrdNum', $language).': '.$orderReplyStatus->gwOrderNumber;
			if ($orderReplyStatus->forexNote !== null) {
				$comment .= "<br/>".$this->uniModul->dictionary->get('forexNoteLabel', $language). ': ' . $orderReplyStatus->forexNote;
			}

			$oldState = $order_info['order_status_id'];
			if ($oldState == 0) {
				if ($orderReplyStatus->orderStatus != OrderStatus::$failedFinal && $orderReplyStatus->orderStatus != OrderStatus::$failedRetriable) {
					$this->model_checkout_order->addHistory($order_id, $newState, $comment, TRUE);
					$clear_cart();
					$this->uniModul->logger->writeLog("ADAPTER: Creating shop order number ".$order_id.", new order state ".$newState);
				} else {
					$this->uniModul->logger->writeLog("ADAPTER: Won't create new failed order " . $order_id);
				}
			} else {
				$transientStates = array();
				if (isset($cfg['orderStatusPending'])) $transientStates[] = $cfg['orderStatusPending'];
				if (isset($cfg['orderStatusFailed'])) $transientStates[] = $cfg['orderStatusFailed'];
				if ($oldState != $newState  && in_array($oldState, $transientStates)) {
					$this->model_checkout_order->addHistory($order_id, $newState, $comment, TRUE);
					$clear_cart();
					$this->uniModul->logger->writeLog("ADAPTER: Updating shop order number ".$order_id.", new order state ".$newState." old order state " . $oldState);
				} else {
					$this->uniModul->logger->writeLog("ADAPTER: Refusing update shop order number ".$order_id.", new order state ".$newState." old order state " . $oldState);
				}

			}
		} else {
			$this->uniModul->logger->writeLog("ADAPTER: Ignoring invalid reply");
		}

		EndSynchronized();

		if ($frontend_redir) {
			if ($guiOk) {
				$clear_cart(); // cart doesn't clear in notifications and callbacks => clear in reply when success
				if  (!empty($orderReplyStatus->successHtml)) {
					$this->load->language('checkout/success');
					$this->document->setTitle($this->language->get('heading_title'));
					$suh = "<h1>".$this->language->get('heading_title')."</h1>";
					$suh .= $orderReplyStatus->successHtml;
					$continue = UniOCHelper::link($this, 'common/home');
					$button_continue = $this->language->get('button_continue');
					$suh .= <<< EOT
	<div class="buttons">
		<div class="right"><a href="$continue" class="button"><span>$button_continue</span></a></div>
	</div>
EOT;
					$this->showInFrontEnd($suh);
				} else {
					UniErr::reset();
					UniOCHelper::redirectLink($this, 'checkout/success');
				}
			} else {
				$this->load->language($this->defaultPath.'uniadapter');
				$errMsg = $this->language->get('message_failure');
				if ($orderReplyStatus->resultText != null) {
					$errMsg .= '<br/>'.$orderReplyStatus->resultText;
				}

				$this->session->data['error'] = $errMsg;
				UniErr::reset();
				UniOCHelper::redirectLink($this, 'checkout/cart');

			}
		}

	}

	/**
	 * @param OrderReplyStatus $orderReplyStatus
	 * @return void
	 */
	public function funcProcessReplyStatus($orderReplyStatus) {
		$this->processReply($orderReplyStatus, false);
	}

	/**
	 * @param string $html
	 * @return void
	 */
	private function showInFrontEnd($html) {
			$data['html_content'] = $html;
			$data['continue_text'] = $this->language->get('button_continue');
			$data['continue_url'] = UniOCHelper::link($this, 'checkout/checkout');

			$data['column_left'] = $this->load->controller('common/column_left');
			$data['column_right'] = $this->load->controller('common/column_right');
			$data['content_top'] = $this->load->controller('common/content_top');
			$data['content_bottom'] = $this->load->controller('common/content_bottom');
			$data['footer'] = $this->load->controller('common/footer');
			$data['header'] = $this->load->controller('common/header');

			$this->response->setOutput($this->load->view($this->defaultPath.'uniadapter_infrontend', $data));
	}
}
