<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\OpenCart;

use Platiti\GPWebPayGpeNs\UniModul\UniModulFactory;
use Platiti\GPWebPayGpeNs\UniModul\UniModul;
use Platiti\GPWebPayGpeNs\UniModul\UniErr;

use \Opencart\System\Engine\Model;
use \Opencart\System\Engine\Registry;
use Exception;
use Throwable;

require_once(__DIR__."/../../../UniOCHelper.php");

class ModelPaymentUniAdapter extends Model {

	/**
	 * @var string[]
	 */
	public static $installmentEmbedHtmlData = [];

	/** @var string */
	protected $adapterName = 'OpenCart';
	/** @var UniModul */
	protected $uniModul;
	/** @var string */
	protected $lowCaseBinderName;
	/** @var string */
	protected $openCartLowCaseBinderName;

	/**
	 * @param Registry $registry
	 * @param string $uniModulName
	 * @param string $subMethod
	 * @return void
	 */
	public function __construct($registry, $uniModulName='', $subMethod='') {
		try {
			parent::__construct($registry);
			UniErr::begin();
			if (VERSION >= '4.0.2.0') {
				$this->lowCaseBinderName = strtolower($uniModulName) . 'binder';
			} else {
				$this->lowCaseBinderName = strtolower($uniModulName) . 'binder' . strtolower($subMethod);
			}
			$this->openCartLowCaseBinderName = UniOCHelper::numberReplace($this->lowCaseBinderName);
			$uniFact = new UniModulFactory();
			$configInfo = $uniFact->getConfigInfo($uniModulName, 'en', $subMethod,  $this->adapterName);
			$ctrlAdapter = $this->getControllerPaymentUniAdapter($this->registry, $uniModulName, $subMethod);
			$this->configSetting = $ctrlAdapter->getConfigData($configInfo, $uniModulName);
			$this->uniModul = $uniFact->createUniModul($uniModulName, $this->configSetting, $subMethod);
			UniErr::end();
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}

	/**
	 * @param string $installmentEmbedHtml
	 *
	 * @return void
	 */
	public function setInstallmentEmbedHtml($installmentEmbedHtml) {
		ModelPaymentUniAdapter::$installmentEmbedHtmlData[$this->uniModul->name] = $installmentEmbedHtml;
	}

	/**
	 * @return string
	 */
	public function getInstallmentEmbedHtml() {
		return ModelPaymentUniAdapter::$installmentEmbedHtmlData[$this->uniModul->name];
	}

	/**
	 * @param array<string, mixed> $address
	 * @return ?array<string, mixed>
	 */
	public function getMethod($address) {
		try {
			if ($this->uniModul===null) return null;

			UniErr::begin();
			if ($this->config->get('payment_' . $this->openCartLowCaseBinderName.'_status')) {
				$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "zone_to_geo_zone WHERE geo_zone_id = '" . (int)$this->config->get('payment_' . $this->openCartLowCaseBinderName.'_geo_zone_id') . "' AND country_id = '" . (int)$address['country_id'] . "' AND (zone_id = '" . (int)$address['zone_id'] . "' OR zone_id = '0')");

				if (!$this->config->get('payment_' . $this->openCartLowCaseBinderName.'_geo_zone_id')) {
					$status = TRUE;
				} elseif ($query->num_rows) {
					$status = TRUE;
				} else {
					$status = FALSE;
				}
			} else {
				$status = FALSE;
			}

			$method_data = array();

			if ($status) {
				$ctrlAdapter = $this->getControllerPaymentUniAdapter($this->registry, $this->uniModul->name, $this->uniModul->subMethod);
				$prePayGWInfo = $ctrlAdapter->getPrePayGWInfoForModel();

				if ($prePayGWInfo->isPossible) {
					$method_data = array(
						'code'       => $this->openCartLowCaseBinderName,
						'title'      => $prePayGWInfo->paymentMethodName,
						'sort_order' => $this->config->get($this->openCartLowCaseBinderName.'_sort_order')
					);
				}
			}

			return UniErr::end($method_data);
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}

	/**
	 * @param array<string, mixed> $address
	 * @return ?array<string, mixed>
	 */
	public function getMethods(array $address = []) {
		try {
			UniErr::begin();
			if ($this->config->get('payment_' . $this->openCartLowCaseBinderName.'_status')) {

				$address = $address ?: $this->session->data["shipping_address"] ?? null;

				if (!$this->config->get('payment_' . $this->openCartLowCaseBinderName.'_geo_zone_id')) {
					$status = TRUE;
				} elseif (isset($address)) {
					$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "zone_to_geo_zone WHERE geo_zone_id = '" . (int)$this->config->get('payment_' . $this->openCartLowCaseBinderName.'_geo_zone_id') . "' AND country_id = '" . (int)$address['country_id'] . "' AND (zone_id = '" . (int)$address['zone_id'] . "' OR zone_id = '0')");
					if ($query->num_rows) {
						$status = TRUE;
					} else {
						$status = FALSE;
					}
				} else {
					$status = FALSE;
					$this->uniModul->logger->writeLog("ADAPTER: WARN cannot check GEO ZONE, no address available for " . $this->lowCaseBinderName);
				}
			} else {
				$status = FALSE;
			}

			$method_data = array();

			if ($status) {
				$ctrlAdapter = $this->getControllerPaymentUniAdapter($this->registry, $this->uniModul->name, '');
				$language = UniOCHelper::getLanguageWoCulture($ctrlAdapter);
				$prePayGWInfo = $ctrlAdapter->getPrePayGWInfoForModel();
				foreach (array_merge([''], $this->uniModul->getSubMethods()) as $subMethod) {
					if ($subMethod == "" || in_array($subMethod, $prePayGWInfo->subMethods ?? [])) {
						if ($this->config->get('payment_' . $this->openCartLowCaseBinderName.'_submethod_'.$subMethod.'_status')) {
							if ($subMethod == '') {
								$option_data[$this->openCartLowCaseBinderName] = [
									'code' => $this->openCartLowCaseBinderName . "." . $this->openCartLowCaseBinderName,
									'name' => $this->uniModul->dictionary->get('payment_method_name', $language)
								];
							} else {
								$option_data[strtolower($this->openCartLowCaseBinderName . $subMethod)] = [
									'code' => $this->openCartLowCaseBinderName . "." . $this->openCartLowCaseBinderName . strtolower($subMethod),
									'name' => $this->uniModul->dictionary->get('submethod_name_' . $subMethod, $language)
								];
							}
						}
					}
				}
				if (isset($option_data)) {
					$method_data = array(
						'code'       => $this->openCartLowCaseBinderName,
						'name'      => $this->uniModul->dictionary->get('payment_method_name', $language),
						'option'     => $option_data,
						'sort_order' => $this->config->get($this->openCartLowCaseBinderName.'_sort_order')
					);
				}
			}

			return UniErr::end($method_data);
		} catch (Exception $ex) {
			UniErr::exception($ex);
		} catch (Throwable $ex) {
			UniErr::exception($ex);
		}
	}

	/**
	 * @param Registry $registry
	 * @param string $uniModulName
	 * @param ?string $subMethod
	 * @return ControllerPaymentUniAdapter
	 */
	protected function getControllerPaymentUniAdapter($registry, $uniModulName, $subMethod) {
		require_once(__DIR__."/../../../catalog/controller/payment/uniadapter.php");
		return new ControllerPaymentUniAdapter($registry, $uniModulName, $subMethod ?? '');
	}
}
