<?php
// Autor (c) Miroslav Novak, www.platiti.cz
// Pouzivani bez souhlasu autora neni povoleno
// #Ver:PRV108-60-g39033a71:2024-08-14#

namespace Platiti\GPWebPayGpeNs\OpenCart;

use Platiti\GPWebPayGpeNs\UniModul\UniModulFactory;
use Platiti\GPWebPayGpeNs\UniModul\UniModul;
use Platiti\GPWebPayGpeNs\UniModul\UniErr;

use Model;
use Exception;
use Throwable;
use Registry;

require_once(__DIR__."/../../../../UniOCHelper.php");

class ModelPaymentUniAdapter extends Model {

	/**
	 * @var string[]
	 */
	public static $installmentEmbedHtmlData = [];

	/** @var string */
	protected $adapterName = 'OpenCart';
	/** @var ?UniModul */
	protected $uniModul;
	/** @var string */
	protected $lowCaseBinderName;

	/**
	 * @param Registry $registry
	 * @param ?string $uniModulName
	 * @param ?string $subMethod
	 * @return void
	 */
	public function __construct($registry, $uniModulName = null, $subMethod='') {
		UniErr::run(null, function() use ($registry, $uniModulName, $subMethod) {
			parent::__construct($registry);
			if ($uniModulName===null) return; //bare adapter, preskocit
			$this->lowCaseBinderName = strtolower($uniModulName).'binder'.strtolower($subMethod);
			$uniFact = new UniModulFactory();
			$configInfo = $uniFact->getConfigInfo($uniModulName, 'en', $subMethod,  $this->adapterName);
			$ctrlAdapter = $this->getControllerPaymentUniAdapter($this->registry, $uniModulName, $subMethod);
			$this->configSetting = $ctrlAdapter->getConfigData($configInfo, $uniModulName);
			$this->uniModul = $uniFact->createUniModul($uniModulName, $this->configSetting, $subMethod);
		});
	}

	/**
	 * @param string $installmentEmbedHtml
	 *
	 * @return void
	 */
	public function setInstallmentEmbedHtml($installmentEmbedHtml) {
		ModelPaymentUniAdapter::$installmentEmbedHtmlData[$this->uniModul->name] = $installmentEmbedHtml;
	}

	/**
	 * @return string
	 */
	public function getInstallmentEmbedHtml() {
		return ModelPaymentUniAdapter::$installmentEmbedHtmlData[$this->uniModul->name];
	}

	/**
	 * @param array<string, mixed> $address
	 * @return ?array<string, mixed>
	 */
	public function getMethod($address) {
		return UniErr::run(null, function() use ($address) {
			if ($this->uniModul===null) return null; // jen protoze adapter je taky modul, TODO; delistovat ho
			if ($this->config->get((VERSION >=3 ? 'payment_' : '') . $this->lowCaseBinderName.'_status')) {
				$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "zone_to_geo_zone WHERE geo_zone_id = '" . (int)$this->config->get((VERSION >=3 ? 'payment_' : '') . $this->lowCaseBinderName.'_geo_zone_id') . "' AND country_id = '" . (int)$address['country_id'] . "' AND (zone_id = '" . (int)$address['zone_id'] . "' OR zone_id = '0')");

				if (!$this->config->get((VERSION >=3 ? 'payment_' : '') . $this->lowCaseBinderName.'_geo_zone_id')) {
					$status = TRUE;
				} elseif ($query->num_rows) {
					$status = TRUE;
				} else {
					$status = FALSE;
				}
			} else {
				$status = FALSE;
			}

			$method_data = array();

			if ($status) {
				$ctrlAdapter = $this->getControllerPaymentUniAdapter($this->registry, $this->uniModul->name, $this->uniModul->subMethod);
				$prePayGWInfo = $ctrlAdapter->getPrePayGWInfoForModel();

				if (isset($GLOBALS['OpenCart_PaymentLogos']) && $GLOBALS['OpenCart_PaymentLogos']!=0) {
					if (file_exists(DIR_APPLICATION."../UniModul/Uni".$this->uniModul->name.$this->uniModul->subMethod."Logo.png")) {
						$submetLogoPiece = $this->uniModul->subMethod;
					} else {
						$submetLogoPiece = "";
					}
					$logoimg = HTTPS_SERVER . "UniModul/Uni".urlencode($this->uniModul->name . $submetLogoPiece)."Logo.png";
					$logoimgel = "<img style='width:7ex;vertical-align:middle' src='".$logoimg."'/>" . " &nbsp; &nbsp; ";
				} else {
					$logoimgel = '';
				}

				if ($prePayGWInfo->isPossible) {
					$method_data = array(
						'id'         => $this->lowCaseBinderName,    //pro ver 1.4
						'code'       => $this->lowCaseBinderName,  //pro ver 1.5
						'title'      => $logoimgel . $prePayGWInfo->paymentMethodName,
						'terms'      => '',
						'sort_order' => $this->config->get($this->lowCaseBinderName.'_sort_order')
					);
				}
			}

			return $method_data;
		});
	}

	/**
	 * @param Registry $registry
	 * @param string $uniModulName
	 * @param ?string $subMethod
	 * @return ControllerPaymentUniAdapter
	 */
	protected function getControllerPaymentUniAdapter($registry, $uniModulName, $subMethod) {
		require_once(__DIR__."/../../../../catalog/controller/extension/payment/uniadapter.php");
		return new ControllerPaymentUniAdapter($registry, $uniModulName, $subMethod);
	}
}
